pub struct ArgumentBuilder<S> { /* private fields */ }Expand description
A node that hasn’t yet been built.
Implementations
Sourceimpl<S> ArgumentBuilder<S>
A node that isn’t yet built.
impl<S> ArgumentBuilder<S>
A node that isn’t yet built.
pub fn new(value: ArgumentBuilderType<S>) -> Self
Sourcepub fn then(self, argument: ArgumentBuilder<S>) -> Self
pub fn then(self, argument: ArgumentBuilder<S>) -> Self
Continue building this node with a child node.
literal("foo").then(literal("bar").executes(|ctx: &CommandContext<()>| 42))Examples found in repository?
azalea/examples/testbot/commands/combat.rs (lines 9-20)
7pub fn register(commands: &mut CommandDispatcher<Mutex<CommandSource>>) {
8 commands.register(
9 literal("killaura").then(argument("enabled", bool()).executes(|ctx: &Ctx| {
10 let enabled = get_bool(ctx, "enabled").unwrap();
11 let source = ctx.source.lock();
12 let bot = source.bot.clone();
13 bot.query_self::<&mut State, _>(|mut state| state.killaura = enabled);
14 source.reply(if enabled {
15 "Enabled killaura"
16 } else {
17 "Disabled killaura"
18 });
19 1
20 })),
21 );
22}More examples
azalea/examples/testbot/commands/movement.rs (lines 31-41)
13pub fn register(commands: &mut CommandDispatcher<Mutex<CommandSource>>) {
14 commands.register(
15 literal("goto")
16 .executes(|ctx: &Ctx| {
17 let source = ctx.source.lock();
18 println!("got goto");
19 // look for the sender
20 let Some(entity) = source.entity() else {
21 source.reply("I can't see you!");
22 return 0;
23 };
24 let position = entity.position();
25 source.reply("ok");
26 source
27 .bot
28 .start_goto(BlockPosGoal(BlockPos::from(position.up(0.5))));
29 1
30 })
31 .then(literal("xz").then(argument("x", integer()).then(
32 argument("z", integer()).executes(|ctx: &Ctx| {
33 let source = ctx.source.lock();
34 let x = get_integer(ctx, "x").unwrap();
35 let z = get_integer(ctx, "z").unwrap();
36 println!("goto xz {x} {z}");
37 source.reply("ok");
38 source.bot.start_goto(XZGoal { x, z });
39 1
40 }),
41 )))
42 .then(literal("radius").then(argument("radius", float()).then(
43 argument("x", integer()).then(argument("y", integer()).then(
44 argument("z", integer()).executes(|ctx: &Ctx| {
45 let source = ctx.source.lock();
46 let radius = get_float(ctx, "radius").unwrap();
47 let x = get_integer(ctx, "x").unwrap();
48 let y = get_integer(ctx, "y").unwrap();
49 let z = get_integer(ctx, "z").unwrap();
50 println!("goto radius {radius}, position: {x} {y} {z}");
51 source.reply("ok");
52 source.bot.start_goto(RadiusGoal {
53 pos: BlockPos::new(x, y, z).center(),
54 radius,
55 });
56 1
57 }),
58 )),
59 )))
60 .then(argument("x", integer()).then(argument("y", integer()).then(
61 argument("z", integer()).executes(|ctx: &Ctx| {
62 let source = ctx.source.lock();
63 let x = get_integer(ctx, "x").unwrap();
64 let y = get_integer(ctx, "y").unwrap();
65 let z = get_integer(ctx, "z").unwrap();
66 println!("goto xyz {x} {y} {z}");
67 source.reply("ok");
68 source.bot.start_goto(BlockPosGoal(BlockPos::new(x, y, z)));
69 1
70 }),
71 ))),
72 );
73
74 commands.register(literal("follow").executes(|ctx: &Ctx| {
75 let source = ctx.source.lock();
76 println!("got follow");
77 // look for the sender
78 let Some(entity) = source.entity() else {
79 source.reply("I can't see you!");
80 return 0;
81 };
82 source.reply("ok");
83 *source.state.following_entity.lock() = Some(entity);
84 1
85 }));
86
87 commands.register(literal("down").executes(|ctx: &Ctx| {
88 let source = ctx.source.clone();
89 tokio::spawn(async move {
90 let bot = source.lock().bot.clone();
91 let position = BlockPos::from(bot.position());
92 source.lock().reply("mining...");
93 bot.mine(position.down(1)).await;
94 source.lock().reply("done");
95 });
96 1
97 }));
98
99 commands.register(
100 literal("look")
101 .executes(|ctx: &Ctx| {
102 // look for the sender
103 let source = ctx.source.lock();
104 let Some(entity) = source.entity() else {
105 source.reply("I can't see you!");
106 return 0;
107 };
108 let eye_position = entity.eye_position();
109 source.bot.look_at(eye_position);
110 1
111 })
112 .then(argument("x", integer()).then(argument("y", integer()).then(
113 argument("z", integer()).executes(|ctx: &Ctx| {
114 let pos = BlockPos::new(
115 get_integer(ctx, "x").unwrap(),
116 get_integer(ctx, "y").unwrap(),
117 get_integer(ctx, "z").unwrap(),
118 );
119 println!("{pos:?}");
120 let source = ctx.source.lock();
121 source.bot.look_at(pos.center());
122 1
123 }),
124 ))),
125 );
126
127 commands.register(
128 literal("walk").then(argument("seconds", float()).executes(|ctx: &Ctx| {
129 let mut seconds = get_float(ctx, "seconds").unwrap();
130 let source = ctx.source.lock();
131 let bot = source.bot.clone();
132
133 if seconds < 0. {
134 bot.walk(WalkDirection::Backward);
135 seconds = -seconds;
136 } else {
137 bot.walk(WalkDirection::Forward);
138 }
139
140 tokio::spawn(async move {
141 tokio::time::sleep(Duration::from_secs_f32(seconds)).await;
142 bot.walk(WalkDirection::None);
143 });
144 source.reply(format!("ok, walking for {seconds} seconds"));
145 1
146 })),
147 );
148 commands.register(
149 literal("sprint").then(argument("seconds", float()).executes(|ctx: &Ctx| {
150 let seconds = get_float(ctx, "seconds").unwrap();
151 let source = ctx.source.lock();
152 let bot = source.bot.clone();
153 bot.sprint(SprintDirection::Forward);
154 tokio::spawn(async move {
155 tokio::time::sleep(Duration::from_secs_f32(seconds)).await;
156 bot.walk(WalkDirection::None);
157 });
158 source.reply(format!("ok, sprinting for {seconds} seconds"));
159 1
160 })),
161 );
162
163 commands.register(literal("north").executes(|ctx: &Ctx| {
164 let source = ctx.source.lock();
165 source.bot.set_direction(180., 0.);
166 source.reply("ok");
167 1
168 }));
169 commands.register(literal("south").executes(|ctx: &Ctx| {
170 let source = ctx.source.lock();
171 source.bot.set_direction(0., 0.);
172 source.reply("ok");
173 1
174 }));
175 commands.register(literal("east").executes(|ctx: &Ctx| {
176 let source = ctx.source.lock();
177 source.bot.set_direction(-90., 0.);
178 source.reply("ok");
179 1
180 }));
181 commands.register(literal("west").executes(|ctx: &Ctx| {
182 let source = ctx.source.lock();
183 source.bot.set_direction(90., 0.);
184 source.reply("ok");
185 1
186 }));
187 commands.register(
188 literal("jump")
189 .executes(|ctx: &Ctx| {
190 let source = ctx.source.lock();
191 source.bot.jump();
192 source.reply("ok");
193 1
194 })
195 .then(argument("enabled", bool()).executes(|ctx: &Ctx| {
196 let jumping = get_bool(ctx, "enabled").unwrap();
197 let source = ctx.source.lock();
198 source.bot.set_jumping(jumping);
199 1
200 })),
201 );
202
203 let sneak = |ctx: &Ctx| {
204 let source = ctx.source.lock();
205 source.bot.set_crouching(!source.bot.crouching());
206 source.reply("ok");
207 1
208 };
209 let sneak_enabled = argument("enabled", bool()).executes(|ctx: &Ctx| {
210 let sneaking = get_bool(ctx, "enabled").unwrap();
211 let source = ctx.source.lock();
212 source.bot.set_crouching(sneaking);
213 1
214 });
215 commands.register(literal("sneak").executes(sneak).then(sneak_enabled.clone()));
216 commands.register(literal("crouch").executes(sneak).then(sneak_enabled));
217
218 commands.register(literal("stop").executes(|ctx: &Ctx| {
219 let source = ctx.source.lock();
220 source.bot.stop_pathfinding();
221 source.reply("ok");
222 *source.state.following_entity.lock() = None;
223 1
224 }));
225 commands.register(literal("forcestop").executes(|ctx: &Ctx| {
226 let source = ctx.source.lock();
227 source.bot.force_stop_pathfinding();
228 source.reply("ok");
229 *source.state.following_entity.lock() = None;
230 1
231 }));
232}azalea/examples/testbot/commands/debug.rs (lines 33-38)
26pub fn register(commands: &mut CommandDispatcher<Mutex<CommandSource>>) {
27 commands.register(literal("ping").executes(|ctx: &Ctx| {
28 let source = ctx.source.lock();
29 source.reply("pong!");
30 1
31 }));
32 commands.register(
33 literal("say").then(argument("message", greedy_string()).executes(|ctx: &Ctx| {
34 let source = ctx.source.lock();
35 let message = get_string(ctx, "message").unwrap();
36 source.bot.chat(message);
37 1
38 })),
39 );
40
41 commands.register(literal("disconnect").executes(|ctx: &Ctx| {
42 let source = ctx.source.lock();
43 source.bot.disconnect();
44 1
45 }));
46
47 commands.register(literal("whereami").executes(|ctx: &Ctx| {
48 let source = ctx.source.lock();
49 let Some(entity) = source.entity() else {
50 source.reply("You aren't in render distance!");
51 return 0;
52 };
53 let position = entity.position();
54 source.reply(format!(
55 "You are at {}, {}, {}",
56 position.x, position.y, position.z
57 ));
58 1
59 }));
60
61 commands.register(literal("entityid").executes(|ctx: &Ctx| {
62 let source = ctx.source.lock();
63 let Some(entity) = source.entity() else {
64 source.reply("You aren't in render distance!");
65 return 0;
66 };
67 let entity_id = entity.minecraft_id();
68 source.reply(format!(
69 "Your Minecraft ID is {} and your ECS ID is {entity:?}",
70 *entity_id
71 ));
72 1
73 }));
74
75 let whereareyou = |ctx: &Ctx| {
76 let source = ctx.source.lock();
77 let position = source.bot.position();
78 source.reply(format!(
79 "I'm at {}, {}, {}",
80 position.x, position.y, position.z
81 ));
82 1
83 };
84 commands.register(literal("whereareyou").executes(whereareyou));
85 commands.register(literal("pos").executes(whereareyou));
86
87 commands.register(literal("whoareyou").executes(|ctx: &Ctx| {
88 let source = ctx.source.lock();
89 source.reply(format!(
90 "I am {} ({}, {})",
91 source.bot.username(),
92 source.bot.uuid(),
93 source.bot.entity
94 ));
95 1
96 }));
97
98 commands.register(literal("getdirection").executes(|ctx: &Ctx| {
99 let source = ctx.source.lock();
100 let direction = source.bot.direction();
101 source.reply(format!(
102 "I'm looking at {}, {}",
103 direction.y_rot(),
104 direction.x_rot()
105 ));
106 1
107 }));
108
109 commands.register(literal("health").executes(|ctx: &Ctx| {
110 let source = ctx.source.lock();
111
112 let health = source.bot.health();
113 source.reply(format!("I have {health} health"));
114 1
115 }));
116
117 commands.register(literal("lookingat").executes(|ctx: &Ctx| {
118 let source = ctx.source.lock();
119
120 let hit_result = source.bot.hit_result();
121
122 match &hit_result {
123 HitResult::Block(r) => {
124 if r.miss {
125 source.reply("I'm not looking at anything");
126 return 0;
127 }
128 let block_pos = r.block_pos;
129 let block = source.bot.world().read().get_block_state(block_pos);
130 source.reply(format!("I'm looking at {block:?} at {block_pos:?}"));
131 }
132 HitResult::Entity(r) => {
133 let entity_kind = **source.bot.entity_component::<EntityKindComponent>(r.entity);
134 source.reply(format!(
135 "I'm looking at {entity_kind} ({:?}) at {}",
136 r.entity, r.location
137 ));
138 }
139 }
140
141 1
142 }));
143
144 commands.register(literal("getblock").then(argument("x", integer()).then(
145 argument("y", integer()).then(argument("z", integer()).executes(|ctx: &Ctx| {
146 let source = ctx.source.lock();
147 let x = get_integer(ctx, "x").unwrap();
148 let y = get_integer(ctx, "y").unwrap();
149 let z = get_integer(ctx, "z").unwrap();
150 println!("getblock xyz {x} {y} {z}");
151 let block_pos = BlockPos::new(x, y, z);
152 let block = source.bot.world().read().get_block_state(block_pos);
153 source.reply(format!("BlockKind at {block_pos} is {block:?}"));
154 1
155 })),
156 )));
157 commands.register(literal("getfluid").then(argument("x", integer()).then(
158 argument("y", integer()).then(argument("z", integer()).executes(|ctx: &Ctx| {
159 let source = ctx.source.lock();
160 let x = get_integer(ctx, "x").unwrap();
161 let y = get_integer(ctx, "y").unwrap();
162 let z = get_integer(ctx, "z").unwrap();
163 println!("getfluid xyz {x} {y} {z}");
164 let block_pos = BlockPos::new(x, y, z);
165 let block = source.bot.world().read().get_fluid_state(block_pos);
166 source.reply(format!("Fluid at {block_pos} is {block:?}"));
167 1
168 })),
169 )));
170
171 commands.register(literal("inventory").executes(|ctx: &Ctx| {
172 let source = ctx.source.lock();
173 for item in source.bot.menu().slots() {
174 if item.is_empty() {
175 continue;
176 }
177 println!("{item:?}");
178 for (kind, data) in item.component_patch().iter() {
179 if let Some(data) = data {
180 println!("- {kind} {data:?}");
181 }
182 }
183 }
184 1
185 }));
186
187 commands.register(literal("pathfinderstate").executes(|ctx: &Ctx| {
188 let source = ctx.source.lock();
189 let pathfinder = source.bot.get_component::<Pathfinder>();
190 let Some(pathfinder) = pathfinder else {
191 source.reply("I don't have the Pathfinder component");
192 return 1;
193 };
194 source.reply(format!(
195 "pathfinder.is_calculating: {}",
196 pathfinder.is_calculating
197 ));
198
199 let executing_path = source.bot.get_component::<ExecutingPath>();
200 let Some(executing_path) = executing_path else {
201 source.reply("I'm not executing a path");
202 return 1;
203 };
204 source.reply(format!(
205 "is_path_partial: {}, path.len: {}, queued_path.len: {}",
206 executing_path.is_path_partial,
207 executing_path.path.len(),
208 if let Some(queued) = &executing_path.queued_path {
209 queued.len().to_string()
210 } else {
211 "n/a".to_owned()
212 },
213 ));
214 1
215 }));
216 commands.register(literal("pathfindermoves").executes(|ctx: &Ctx| {
217 let source = ctx.source.lock();
218
219 let Some(entity) = source.entity() else {
220 source.reply("You aren't in render distance!");
221 return 0;
222 };
223 let position = entity.position();
224 let position = BlockPos::from(position);
225
226 let mut edges = Vec::new();
227 let cached_world = CachedWorld::new(source.bot.world(), position);
228 let mining_cache = MiningCache::new(Some(Menu::Player(inventory::Player::default())));
229 let custom_state = CustomPathfinderStateRef::default();
230
231 azalea::pathfinder::moves::default_move(
232 &mut MovesCtx {
233 edges: &mut edges,
234 world: &cached_world,
235 mining_cache: &mining_cache,
236 custom_state: &custom_state,
237 },
238 RelBlockPos::from_origin(position, position),
239 );
240
241 if edges.is_empty() {
242 source.reply("No possible moves.");
243 } else {
244 source.reply("Moves:");
245 for (i, edge) in edges.iter().enumerate() {
246 source.reply(format!("{}) {edge:?}", i + 1));
247 }
248 }
249
250 1
251 }));
252
253 commands.register(literal("startuseitem").executes(|ctx: &Ctx| {
254 let source = ctx.source.lock();
255 source.bot.start_use_item();
256 source.reply("Ok!");
257 1
258 }));
259 commands.register(literal("maxstacksize").executes(|ctx: &Ctx| {
260 let source = ctx.source.lock();
261 let max_stack_size = source
262 .bot
263 .get_held_item()
264 .get_component::<MaxStackSize>()
265 .map_or(-1, |s| s.count);
266 source.reply(format!("{max_stack_size}"));
267 1
268 }));
269
270 commands.register(literal("dimensions").executes(|ctx: &Ctx| {
271 let source = ctx.source.lock();
272 let bot_dimensions = source.bot.dimensions();
273 source.reply(format!("{bot_dimensions:?}"));
274 1
275 }));
276
277 commands.register(literal("players").executes(|ctx: &Ctx| {
278 let source = ctx.source.lock();
279 let player_entities = source
280 .bot
281 .nearest_entities_by::<(), With<metadata::Player>>(|_: ()| true);
282 let tab_list = source.bot.tab_list();
283 for player_entity in player_entities {
284 let uuid = player_entity.uuid();
285 source.reply(format!(
286 "{} - {} ({:?})",
287 player_entity.id(),
288 tab_list.get(&uuid).map_or("?", |p| p.profile.name.as_str()),
289 uuid
290 ));
291 }
292 1
293 }));
294
295 commands.register(literal("enchants").executes(|ctx: &Ctx| {
296 let source = ctx.source.lock();
297 source.bot.with_registry_holder(|r| {
298 let enchants = &r.enchantment;
299 println!("enchants: {enchants:?}");
300 });
301 1
302 }));
303
304 commands.register(literal("attributes").executes(|ctx: &Ctx| {
305 let source = ctx.source.lock();
306 let attributes = source.bot.attributes();
307 println!("attributes: {attributes:?}");
308 1
309 }));
310
311 commands.register(literal("debugecsleak").executes(|ctx: &Ctx| {
312 let source = ctx.source.lock();
313
314 source.reply("Ok!");
315
316
317
318 source.bot.disconnect();
319
320 let ecs = source.bot.ecs.clone();
321 thread::spawn(move || {
322 thread::sleep(Duration::from_secs(1));
323 // dump the ecs
324
325 let mut ecs = ecs.write();
326
327 let report_path = env::temp_dir().join("azalea-ecs-leak-report.txt");
328 let mut report = File::create(&report_path).unwrap();
329
330 let mut query = ecs.query::<EntityRef>();
331 for entity in query.iter(& ecs) {
332 writeln!(report, "Entity: {}", entity.id()).unwrap();
333 let archetype = entity.archetype();
334 let component_count = archetype.component_count();
335
336 let component_names = archetype
337 .components()
338 .iter()
339 .map(|c| ecs.components().get_info(*c).unwrap().name().to_string())
340 .collect::<Vec<_>>();
341 writeln!(
342 report,
343 "- {component_count} components: {}",
344 component_names.join(", ")
345 )
346 .unwrap();
347 }
348
349 writeln!(report).unwrap();
350
351
352 for (info, _) in ecs.iter_resources() {
353 let name = info.name().to_string();
354 writeln!(report, "Resource: {name}").unwrap();
355 // writeln!(report, "- Size: {} bytes",
356 // info.layout().size()).unwrap();
357
358 match name.as_ref() {
359 "azalea_world::container::Worlds" => {
360 let worlds = ecs.resource::<Worlds>();
361
362 for (world_name, world) in &worlds.map {
363 writeln!(report, "- Name: {world_name}").unwrap();
364 writeln!(report, "- Reference count: {}", world.strong_count())
365 .unwrap();
366 if let Some(world) = world.upgrade() {
367 let world = world.read();
368 let chunks = &world.chunks;
369 let chunks = (chunks as &dyn Any).downcast_ref::<WeakChunkStorage>();
370 if let Some(chunks) = chunks {
371 let strong_chunks = chunks
372 .map
373 .iter()
374 .filter(|(_, v)| v.strong_count() > 0)
375 .count();
376 writeln!(
377 report,
378 "- Chunks: {} strongly referenced, {} in map",
379 strong_chunks,
380 chunks.map.len()
381 )
382 .unwrap();
383 }
384 writeln!(
385 report,
386 "- Entities: {}",
387 world.entities_by_chunk.len()
388 )
389 .unwrap();
390 }
391 }
392 }
393 "bevy_ecs::message::Messages<azalea_client::packet::game::ReceivePacketEvent>" => {
394 let events = ecs.resource::<Messages<game::ReceiveGamePacketEvent>>();
395 writeln!(report, "- Event count: {}", events.len()).unwrap();
396 }
397 "bevy_ecs::message::Messages<azalea_client::chunks::ReceiveChunkEvent>" => {
398 let events = ecs.resource::<Messages<ReceiveChunkEvent>>();
399 writeln!(report, "- Event count: {}", events.len()).unwrap();
400 }
401
402 _ => {}
403 }
404 }
405
406 println!("\x1b[1mWrote report to {}\x1b[m", report_path.display());
407 });
408
409 1
410 }));
411
412 commands.register(literal("exit").executes(|ctx: &Ctx| {
413 let source = ctx.source.lock();
414 source.reply("bye!");
415
416 source.bot.disconnect();
417
418 let source = ctx.source.clone();
419 thread::spawn(move || {
420 thread::sleep(Duration::from_secs(1));
421
422 source
423 .lock()
424 .bot
425 .ecs
426 .write()
427 .write_message(AppExit::Success);
428 });
429
430 1
431 }));
432}Sourcepub fn then_built(self, argument: CommandNode<S>) -> Self
pub fn then_built(self, argument: CommandNode<S>) -> Self
Add an already built child node to this node.
You should usually use Self::then instead.
Sourcepub fn executes<F>(self, f: F) -> Self
pub fn executes<F>(self, f: F) -> Self
Set the command to be executed when this node is reached.
If this is not present on a node, it is not a valid command.
literal("foo").executes(|ctx: &CommandContext<()>| 42)Examples found in repository?
azalea/examples/testbot/commands/combat.rs (lines 9-20)
7pub fn register(commands: &mut CommandDispatcher<Mutex<CommandSource>>) {
8 commands.register(
9 literal("killaura").then(argument("enabled", bool()).executes(|ctx: &Ctx| {
10 let enabled = get_bool(ctx, "enabled").unwrap();
11 let source = ctx.source.lock();
12 let bot = source.bot.clone();
13 bot.query_self::<&mut State, _>(|mut state| state.killaura = enabled);
14 source.reply(if enabled {
15 "Enabled killaura"
16 } else {
17 "Disabled killaura"
18 });
19 1
20 })),
21 );
22}More examples
azalea/examples/testbot/commands/movement.rs (lines 16-30)
13pub fn register(commands: &mut CommandDispatcher<Mutex<CommandSource>>) {
14 commands.register(
15 literal("goto")
16 .executes(|ctx: &Ctx| {
17 let source = ctx.source.lock();
18 println!("got goto");
19 // look for the sender
20 let Some(entity) = source.entity() else {
21 source.reply("I can't see you!");
22 return 0;
23 };
24 let position = entity.position();
25 source.reply("ok");
26 source
27 .bot
28 .start_goto(BlockPosGoal(BlockPos::from(position.up(0.5))));
29 1
30 })
31 .then(literal("xz").then(argument("x", integer()).then(
32 argument("z", integer()).executes(|ctx: &Ctx| {
33 let source = ctx.source.lock();
34 let x = get_integer(ctx, "x").unwrap();
35 let z = get_integer(ctx, "z").unwrap();
36 println!("goto xz {x} {z}");
37 source.reply("ok");
38 source.bot.start_goto(XZGoal { x, z });
39 1
40 }),
41 )))
42 .then(literal("radius").then(argument("radius", float()).then(
43 argument("x", integer()).then(argument("y", integer()).then(
44 argument("z", integer()).executes(|ctx: &Ctx| {
45 let source = ctx.source.lock();
46 let radius = get_float(ctx, "radius").unwrap();
47 let x = get_integer(ctx, "x").unwrap();
48 let y = get_integer(ctx, "y").unwrap();
49 let z = get_integer(ctx, "z").unwrap();
50 println!("goto radius {radius}, position: {x} {y} {z}");
51 source.reply("ok");
52 source.bot.start_goto(RadiusGoal {
53 pos: BlockPos::new(x, y, z).center(),
54 radius,
55 });
56 1
57 }),
58 )),
59 )))
60 .then(argument("x", integer()).then(argument("y", integer()).then(
61 argument("z", integer()).executes(|ctx: &Ctx| {
62 let source = ctx.source.lock();
63 let x = get_integer(ctx, "x").unwrap();
64 let y = get_integer(ctx, "y").unwrap();
65 let z = get_integer(ctx, "z").unwrap();
66 println!("goto xyz {x} {y} {z}");
67 source.reply("ok");
68 source.bot.start_goto(BlockPosGoal(BlockPos::new(x, y, z)));
69 1
70 }),
71 ))),
72 );
73
74 commands.register(literal("follow").executes(|ctx: &Ctx| {
75 let source = ctx.source.lock();
76 println!("got follow");
77 // look for the sender
78 let Some(entity) = source.entity() else {
79 source.reply("I can't see you!");
80 return 0;
81 };
82 source.reply("ok");
83 *source.state.following_entity.lock() = Some(entity);
84 1
85 }));
86
87 commands.register(literal("down").executes(|ctx: &Ctx| {
88 let source = ctx.source.clone();
89 tokio::spawn(async move {
90 let bot = source.lock().bot.clone();
91 let position = BlockPos::from(bot.position());
92 source.lock().reply("mining...");
93 bot.mine(position.down(1)).await;
94 source.lock().reply("done");
95 });
96 1
97 }));
98
99 commands.register(
100 literal("look")
101 .executes(|ctx: &Ctx| {
102 // look for the sender
103 let source = ctx.source.lock();
104 let Some(entity) = source.entity() else {
105 source.reply("I can't see you!");
106 return 0;
107 };
108 let eye_position = entity.eye_position();
109 source.bot.look_at(eye_position);
110 1
111 })
112 .then(argument("x", integer()).then(argument("y", integer()).then(
113 argument("z", integer()).executes(|ctx: &Ctx| {
114 let pos = BlockPos::new(
115 get_integer(ctx, "x").unwrap(),
116 get_integer(ctx, "y").unwrap(),
117 get_integer(ctx, "z").unwrap(),
118 );
119 println!("{pos:?}");
120 let source = ctx.source.lock();
121 source.bot.look_at(pos.center());
122 1
123 }),
124 ))),
125 );
126
127 commands.register(
128 literal("walk").then(argument("seconds", float()).executes(|ctx: &Ctx| {
129 let mut seconds = get_float(ctx, "seconds").unwrap();
130 let source = ctx.source.lock();
131 let bot = source.bot.clone();
132
133 if seconds < 0. {
134 bot.walk(WalkDirection::Backward);
135 seconds = -seconds;
136 } else {
137 bot.walk(WalkDirection::Forward);
138 }
139
140 tokio::spawn(async move {
141 tokio::time::sleep(Duration::from_secs_f32(seconds)).await;
142 bot.walk(WalkDirection::None);
143 });
144 source.reply(format!("ok, walking for {seconds} seconds"));
145 1
146 })),
147 );
148 commands.register(
149 literal("sprint").then(argument("seconds", float()).executes(|ctx: &Ctx| {
150 let seconds = get_float(ctx, "seconds").unwrap();
151 let source = ctx.source.lock();
152 let bot = source.bot.clone();
153 bot.sprint(SprintDirection::Forward);
154 tokio::spawn(async move {
155 tokio::time::sleep(Duration::from_secs_f32(seconds)).await;
156 bot.walk(WalkDirection::None);
157 });
158 source.reply(format!("ok, sprinting for {seconds} seconds"));
159 1
160 })),
161 );
162
163 commands.register(literal("north").executes(|ctx: &Ctx| {
164 let source = ctx.source.lock();
165 source.bot.set_direction(180., 0.);
166 source.reply("ok");
167 1
168 }));
169 commands.register(literal("south").executes(|ctx: &Ctx| {
170 let source = ctx.source.lock();
171 source.bot.set_direction(0., 0.);
172 source.reply("ok");
173 1
174 }));
175 commands.register(literal("east").executes(|ctx: &Ctx| {
176 let source = ctx.source.lock();
177 source.bot.set_direction(-90., 0.);
178 source.reply("ok");
179 1
180 }));
181 commands.register(literal("west").executes(|ctx: &Ctx| {
182 let source = ctx.source.lock();
183 source.bot.set_direction(90., 0.);
184 source.reply("ok");
185 1
186 }));
187 commands.register(
188 literal("jump")
189 .executes(|ctx: &Ctx| {
190 let source = ctx.source.lock();
191 source.bot.jump();
192 source.reply("ok");
193 1
194 })
195 .then(argument("enabled", bool()).executes(|ctx: &Ctx| {
196 let jumping = get_bool(ctx, "enabled").unwrap();
197 let source = ctx.source.lock();
198 source.bot.set_jumping(jumping);
199 1
200 })),
201 );
202
203 let sneak = |ctx: &Ctx| {
204 let source = ctx.source.lock();
205 source.bot.set_crouching(!source.bot.crouching());
206 source.reply("ok");
207 1
208 };
209 let sneak_enabled = argument("enabled", bool()).executes(|ctx: &Ctx| {
210 let sneaking = get_bool(ctx, "enabled").unwrap();
211 let source = ctx.source.lock();
212 source.bot.set_crouching(sneaking);
213 1
214 });
215 commands.register(literal("sneak").executes(sneak).then(sneak_enabled.clone()));
216 commands.register(literal("crouch").executes(sneak).then(sneak_enabled));
217
218 commands.register(literal("stop").executes(|ctx: &Ctx| {
219 let source = ctx.source.lock();
220 source.bot.stop_pathfinding();
221 source.reply("ok");
222 *source.state.following_entity.lock() = None;
223 1
224 }));
225 commands.register(literal("forcestop").executes(|ctx: &Ctx| {
226 let source = ctx.source.lock();
227 source.bot.force_stop_pathfinding();
228 source.reply("ok");
229 *source.state.following_entity.lock() = None;
230 1
231 }));
232}azalea/examples/testbot/commands/debug.rs (lines 27-31)
26pub fn register(commands: &mut CommandDispatcher<Mutex<CommandSource>>) {
27 commands.register(literal("ping").executes(|ctx: &Ctx| {
28 let source = ctx.source.lock();
29 source.reply("pong!");
30 1
31 }));
32 commands.register(
33 literal("say").then(argument("message", greedy_string()).executes(|ctx: &Ctx| {
34 let source = ctx.source.lock();
35 let message = get_string(ctx, "message").unwrap();
36 source.bot.chat(message);
37 1
38 })),
39 );
40
41 commands.register(literal("disconnect").executes(|ctx: &Ctx| {
42 let source = ctx.source.lock();
43 source.bot.disconnect();
44 1
45 }));
46
47 commands.register(literal("whereami").executes(|ctx: &Ctx| {
48 let source = ctx.source.lock();
49 let Some(entity) = source.entity() else {
50 source.reply("You aren't in render distance!");
51 return 0;
52 };
53 let position = entity.position();
54 source.reply(format!(
55 "You are at {}, {}, {}",
56 position.x, position.y, position.z
57 ));
58 1
59 }));
60
61 commands.register(literal("entityid").executes(|ctx: &Ctx| {
62 let source = ctx.source.lock();
63 let Some(entity) = source.entity() else {
64 source.reply("You aren't in render distance!");
65 return 0;
66 };
67 let entity_id = entity.minecraft_id();
68 source.reply(format!(
69 "Your Minecraft ID is {} and your ECS ID is {entity:?}",
70 *entity_id
71 ));
72 1
73 }));
74
75 let whereareyou = |ctx: &Ctx| {
76 let source = ctx.source.lock();
77 let position = source.bot.position();
78 source.reply(format!(
79 "I'm at {}, {}, {}",
80 position.x, position.y, position.z
81 ));
82 1
83 };
84 commands.register(literal("whereareyou").executes(whereareyou));
85 commands.register(literal("pos").executes(whereareyou));
86
87 commands.register(literal("whoareyou").executes(|ctx: &Ctx| {
88 let source = ctx.source.lock();
89 source.reply(format!(
90 "I am {} ({}, {})",
91 source.bot.username(),
92 source.bot.uuid(),
93 source.bot.entity
94 ));
95 1
96 }));
97
98 commands.register(literal("getdirection").executes(|ctx: &Ctx| {
99 let source = ctx.source.lock();
100 let direction = source.bot.direction();
101 source.reply(format!(
102 "I'm looking at {}, {}",
103 direction.y_rot(),
104 direction.x_rot()
105 ));
106 1
107 }));
108
109 commands.register(literal("health").executes(|ctx: &Ctx| {
110 let source = ctx.source.lock();
111
112 let health = source.bot.health();
113 source.reply(format!("I have {health} health"));
114 1
115 }));
116
117 commands.register(literal("lookingat").executes(|ctx: &Ctx| {
118 let source = ctx.source.lock();
119
120 let hit_result = source.bot.hit_result();
121
122 match &hit_result {
123 HitResult::Block(r) => {
124 if r.miss {
125 source.reply("I'm not looking at anything");
126 return 0;
127 }
128 let block_pos = r.block_pos;
129 let block = source.bot.world().read().get_block_state(block_pos);
130 source.reply(format!("I'm looking at {block:?} at {block_pos:?}"));
131 }
132 HitResult::Entity(r) => {
133 let entity_kind = **source.bot.entity_component::<EntityKindComponent>(r.entity);
134 source.reply(format!(
135 "I'm looking at {entity_kind} ({:?}) at {}",
136 r.entity, r.location
137 ));
138 }
139 }
140
141 1
142 }));
143
144 commands.register(literal("getblock").then(argument("x", integer()).then(
145 argument("y", integer()).then(argument("z", integer()).executes(|ctx: &Ctx| {
146 let source = ctx.source.lock();
147 let x = get_integer(ctx, "x").unwrap();
148 let y = get_integer(ctx, "y").unwrap();
149 let z = get_integer(ctx, "z").unwrap();
150 println!("getblock xyz {x} {y} {z}");
151 let block_pos = BlockPos::new(x, y, z);
152 let block = source.bot.world().read().get_block_state(block_pos);
153 source.reply(format!("BlockKind at {block_pos} is {block:?}"));
154 1
155 })),
156 )));
157 commands.register(literal("getfluid").then(argument("x", integer()).then(
158 argument("y", integer()).then(argument("z", integer()).executes(|ctx: &Ctx| {
159 let source = ctx.source.lock();
160 let x = get_integer(ctx, "x").unwrap();
161 let y = get_integer(ctx, "y").unwrap();
162 let z = get_integer(ctx, "z").unwrap();
163 println!("getfluid xyz {x} {y} {z}");
164 let block_pos = BlockPos::new(x, y, z);
165 let block = source.bot.world().read().get_fluid_state(block_pos);
166 source.reply(format!("Fluid at {block_pos} is {block:?}"));
167 1
168 })),
169 )));
170
171 commands.register(literal("inventory").executes(|ctx: &Ctx| {
172 let source = ctx.source.lock();
173 for item in source.bot.menu().slots() {
174 if item.is_empty() {
175 continue;
176 }
177 println!("{item:?}");
178 for (kind, data) in item.component_patch().iter() {
179 if let Some(data) = data {
180 println!("- {kind} {data:?}");
181 }
182 }
183 }
184 1
185 }));
186
187 commands.register(literal("pathfinderstate").executes(|ctx: &Ctx| {
188 let source = ctx.source.lock();
189 let pathfinder = source.bot.get_component::<Pathfinder>();
190 let Some(pathfinder) = pathfinder else {
191 source.reply("I don't have the Pathfinder component");
192 return 1;
193 };
194 source.reply(format!(
195 "pathfinder.is_calculating: {}",
196 pathfinder.is_calculating
197 ));
198
199 let executing_path = source.bot.get_component::<ExecutingPath>();
200 let Some(executing_path) = executing_path else {
201 source.reply("I'm not executing a path");
202 return 1;
203 };
204 source.reply(format!(
205 "is_path_partial: {}, path.len: {}, queued_path.len: {}",
206 executing_path.is_path_partial,
207 executing_path.path.len(),
208 if let Some(queued) = &executing_path.queued_path {
209 queued.len().to_string()
210 } else {
211 "n/a".to_owned()
212 },
213 ));
214 1
215 }));
216 commands.register(literal("pathfindermoves").executes(|ctx: &Ctx| {
217 let source = ctx.source.lock();
218
219 let Some(entity) = source.entity() else {
220 source.reply("You aren't in render distance!");
221 return 0;
222 };
223 let position = entity.position();
224 let position = BlockPos::from(position);
225
226 let mut edges = Vec::new();
227 let cached_world = CachedWorld::new(source.bot.world(), position);
228 let mining_cache = MiningCache::new(Some(Menu::Player(inventory::Player::default())));
229 let custom_state = CustomPathfinderStateRef::default();
230
231 azalea::pathfinder::moves::default_move(
232 &mut MovesCtx {
233 edges: &mut edges,
234 world: &cached_world,
235 mining_cache: &mining_cache,
236 custom_state: &custom_state,
237 },
238 RelBlockPos::from_origin(position, position),
239 );
240
241 if edges.is_empty() {
242 source.reply("No possible moves.");
243 } else {
244 source.reply("Moves:");
245 for (i, edge) in edges.iter().enumerate() {
246 source.reply(format!("{}) {edge:?}", i + 1));
247 }
248 }
249
250 1
251 }));
252
253 commands.register(literal("startuseitem").executes(|ctx: &Ctx| {
254 let source = ctx.source.lock();
255 source.bot.start_use_item();
256 source.reply("Ok!");
257 1
258 }));
259 commands.register(literal("maxstacksize").executes(|ctx: &Ctx| {
260 let source = ctx.source.lock();
261 let max_stack_size = source
262 .bot
263 .get_held_item()
264 .get_component::<MaxStackSize>()
265 .map_or(-1, |s| s.count);
266 source.reply(format!("{max_stack_size}"));
267 1
268 }));
269
270 commands.register(literal("dimensions").executes(|ctx: &Ctx| {
271 let source = ctx.source.lock();
272 let bot_dimensions = source.bot.dimensions();
273 source.reply(format!("{bot_dimensions:?}"));
274 1
275 }));
276
277 commands.register(literal("players").executes(|ctx: &Ctx| {
278 let source = ctx.source.lock();
279 let player_entities = source
280 .bot
281 .nearest_entities_by::<(), With<metadata::Player>>(|_: ()| true);
282 let tab_list = source.bot.tab_list();
283 for player_entity in player_entities {
284 let uuid = player_entity.uuid();
285 source.reply(format!(
286 "{} - {} ({:?})",
287 player_entity.id(),
288 tab_list.get(&uuid).map_or("?", |p| p.profile.name.as_str()),
289 uuid
290 ));
291 }
292 1
293 }));
294
295 commands.register(literal("enchants").executes(|ctx: &Ctx| {
296 let source = ctx.source.lock();
297 source.bot.with_registry_holder(|r| {
298 let enchants = &r.enchantment;
299 println!("enchants: {enchants:?}");
300 });
301 1
302 }));
303
304 commands.register(literal("attributes").executes(|ctx: &Ctx| {
305 let source = ctx.source.lock();
306 let attributes = source.bot.attributes();
307 println!("attributes: {attributes:?}");
308 1
309 }));
310
311 commands.register(literal("debugecsleak").executes(|ctx: &Ctx| {
312 let source = ctx.source.lock();
313
314 source.reply("Ok!");
315
316
317
318 source.bot.disconnect();
319
320 let ecs = source.bot.ecs.clone();
321 thread::spawn(move || {
322 thread::sleep(Duration::from_secs(1));
323 // dump the ecs
324
325 let mut ecs = ecs.write();
326
327 let report_path = env::temp_dir().join("azalea-ecs-leak-report.txt");
328 let mut report = File::create(&report_path).unwrap();
329
330 let mut query = ecs.query::<EntityRef>();
331 for entity in query.iter(& ecs) {
332 writeln!(report, "Entity: {}", entity.id()).unwrap();
333 let archetype = entity.archetype();
334 let component_count = archetype.component_count();
335
336 let component_names = archetype
337 .components()
338 .iter()
339 .map(|c| ecs.components().get_info(*c).unwrap().name().to_string())
340 .collect::<Vec<_>>();
341 writeln!(
342 report,
343 "- {component_count} components: {}",
344 component_names.join(", ")
345 )
346 .unwrap();
347 }
348
349 writeln!(report).unwrap();
350
351
352 for (info, _) in ecs.iter_resources() {
353 let name = info.name().to_string();
354 writeln!(report, "Resource: {name}").unwrap();
355 // writeln!(report, "- Size: {} bytes",
356 // info.layout().size()).unwrap();
357
358 match name.as_ref() {
359 "azalea_world::container::Worlds" => {
360 let worlds = ecs.resource::<Worlds>();
361
362 for (world_name, world) in &worlds.map {
363 writeln!(report, "- Name: {world_name}").unwrap();
364 writeln!(report, "- Reference count: {}", world.strong_count())
365 .unwrap();
366 if let Some(world) = world.upgrade() {
367 let world = world.read();
368 let chunks = &world.chunks;
369 let chunks = (chunks as &dyn Any).downcast_ref::<WeakChunkStorage>();
370 if let Some(chunks) = chunks {
371 let strong_chunks = chunks
372 .map
373 .iter()
374 .filter(|(_, v)| v.strong_count() > 0)
375 .count();
376 writeln!(
377 report,
378 "- Chunks: {} strongly referenced, {} in map",
379 strong_chunks,
380 chunks.map.len()
381 )
382 .unwrap();
383 }
384 writeln!(
385 report,
386 "- Entities: {}",
387 world.entities_by_chunk.len()
388 )
389 .unwrap();
390 }
391 }
392 }
393 "bevy_ecs::message::Messages<azalea_client::packet::game::ReceivePacketEvent>" => {
394 let events = ecs.resource::<Messages<game::ReceiveGamePacketEvent>>();
395 writeln!(report, "- Event count: {}", events.len()).unwrap();
396 }
397 "bevy_ecs::message::Messages<azalea_client::chunks::ReceiveChunkEvent>" => {
398 let events = ecs.resource::<Messages<ReceiveChunkEvent>>();
399 writeln!(report, "- Event count: {}", events.len()).unwrap();
400 }
401
402 _ => {}
403 }
404 }
405
406 println!("\x1b[1mWrote report to {}\x1b[m", report_path.display());
407 });
408
409 1
410 }));
411
412 commands.register(literal("exit").executes(|ctx: &Ctx| {
413 let source = ctx.source.lock();
414 source.reply("bye!");
415
416 source.bot.disconnect();
417
418 let source = ctx.source.clone();
419 thread::spawn(move || {
420 thread::sleep(Duration::from_secs(1));
421
422 source
423 .lock()
424 .bot
425 .ecs
426 .write()
427 .write_message(AppExit::Success);
428 });
429
430 1
431 }));
432}Sourcepub fn executes_result<F>(self, f: F) -> Self
pub fn executes_result<F>(self, f: F) -> Self
Same as Self::executes but returns a Result<i32, CommandSyntaxError>.
Sourcepub fn requires<F>(self, requirement: F) -> Self
pub fn requires<F>(self, requirement: F) -> Self
Set the requirement for this node to be considered.
If this is not present on a node, it is considered to always pass.
literal("foo")
.requires(|s: &CommandSource| s.opped)
// ...pub fn redirect(self, target: Arc<RwLock<CommandNode<S>>>) -> Self
pub fn fork( self, target: Arc<RwLock<CommandNode<S>>>, modifier: Arc<RedirectModifier<S>>, ) -> Self
pub fn forward( self, target: Arc<RwLock<CommandNode<S>>>, modifier: Option<Arc<RedirectModifier<S>>>, fork: bool, ) -> Self
pub fn arguments(&self) -> &CommandNode<S>
Sourcepub fn build(self) -> CommandNode<S>
pub fn build(self) -> CommandNode<S>
Manually build this node into a CommandNode. You probably don’t need
to do this yourself.